/*
   core/slot.cc
   This file is part of the Osirion project and is distributed under
   the terms of the GNU General Public License version 2.
*/

#include "core/slot.h"
#include "core/item.h"
#include "core/application.h"

namespace core
{

Slot::Slot()
{
	clear();
}

Slot::~Slot()
{
}

void Slot::load(const model::Slot *slot_tag, const float modelscale)
{
	if (slot_tag) {
		set_type(slot_tag->type());
		set_location(slot_tag->location() * modelscale);
		set_axis(slot_tag->axis());
		set_radius(slot_tag->radius());
		set_cone(slot_tag->cone() * M_PI  / 180.0f);
	} else {
		Slot();
	}
}

void Slot::clear()
{
	slot_item = 0;
	slot_flags = 0;
	slot_timestamp = 0;
	slot_last_fired = 0;
	slot_cone = 0;
	slot_radius = 1.0f;
	slot_type = model::Slot::None;
}

void Slot::set_flag(const Flags flag)
{
	slot_flags = (slot_flags | (unsigned int) flag);
	set_timestamp(game() ? game()->timestamp() : 1);
}

void Slot::unset_flag(const Flags flag)
{
	slot_flags = slot_flags & ~((unsigned int) flag);
	set_timestamp(game() ? game()->timestamp() : 1);
}

void Slot::set_item(Item *item)
{
	slot_item = item;
	set_timestamp(game() ? game()->timestamp() : 1);
}

} // namespace core
