/*
   model/layer.cc
   This file is part of the Osirion project and is distributed under
   the terms of the GNU General Public License version 2
*/

#include "model/layer.h"

namespace model
{

Layer::Layer() :
	layer_size(),
	layer_rgbgen(RGBGenIdentity),
	layer_color(1.0f),
	layer_color_specular(1.0f),
	layer_shininess(32),
	layer_texmap(TexMapNone),
	layer_texture_name(),
	layer_texture_id(0),
	layer_tcgen(TCGenBase),
	layer_fullbright(false),
	layer_blendfunc(BlendFuncNone)
{
}

Layer::~Layer()
{
}

void Layer::set_size(const math::Vector2f & size)
{
	layer_size.assign(size);
}

void Layer::set_rgbgen(const RGBGen rgbgen)
{
	layer_rgbgen = rgbgen;
}
	
void Layer::set_color(const math::Color & color)
{
	layer_color.assign(color);
}

void Layer::set_specular(const math::Color & specular)
{
	layer_color_specular.assign(specular);
}

void Layer::set_shininess(const math::Color & shininess)
{
	layer_shininess.assign(shininess);
}

void Layer::set_texmap(const TexMap texmap)
{
	layer_texmap = texmap;
}

void Layer::set_texture(const std::string & texture)
{
	if (texture.size())
	{
		layer_texture_name.assign(texture);
		set_texmap(TexMapImage);
	} else {
		layer_texture_name.clear();
		set_texmap(TexMapNone);
	}	
}
	
void Layer::set_texture_id(const size_t id)
{
	layer_texture_id = id;
}

void Layer::set_tcgen(const TCGen tcgen)
{
	layer_tcgen = tcgen;
}

void Layer::set_fullbright(const bool fullbright)
{
	layer_fullbright = fullbright;
}

void Layer::set_blendfunc(const BlendFunc blendfunc)
{
	layer_blendfunc = blendfunc;
}

}

