/*
   ui/paint.h
   This file is part of the Osirion project and is distributed under
   the terms of the GNU General Public License version 2
*/

#ifndef __INCLUDED_UI_PAINT_H__
#define __INCLUDED_UI_PAINT_H__

#include "ui/widget.h"
#include "model/material.h"

namespace ui {

/**
 * @brief low-level widget paint functions
 * This class contains the interface between the user interface and the render library
 */

class Paint
{
public:
	/**
	 * @brief set paint color to RGB value
	 * */
	static void set_color(float r = 0.0f, float g = 0.0f, float b = 0.0f, float a = 1.0f);

	/**
	 * @brief set paint color to RGB value
	 * */
	static void set_color(math::Color const & color);

	/**
	 * @brief set paint color to system color code
	 * */
	static void set_system_color(const char c);

	/**
	 * @brief assign system color code
	 * */
	static void assign_system_color(const char c, const math::Color &color);

	/**
	 * @brief draw a border
	 * */
	static void draw_border(const math::Vector2f &global_location, const math::Vector2f &size);

	/**
	 * @brief draw a rectangle
	 * */
	static void draw_rectangle(const math::Vector2f &global_location, const math::Vector2f &size);
	
	/**
	 * @brief draw a color gradient rectangle
	 * */
	static void draw_rectangle_gradient(const math::Vector2f &global_location, const math::Vector2f &size, const math::Color & color_left, const math::Color & color_right);
	
	/**
	 * @brief draw a rectangular bitmap
	 * @param texture the name of material to be used
	 **/
	static void draw_bitmap(const math::Vector2f &global_location, const math::Vector2f &size, const std::string &texture, const float preserve_aspect = false);
	
	/**
	 * @brief draw a rectangular bitmap and overrride material color
	 * */
	static void draw_bitmap(const math::Vector2f &global_location, const math::Vector2f &size, const math::Color & color, const std::string &texture, const float preserve_aspect = false);
	
	/**
	 * @brief draw a tiled bitmap
	 * */
	static void draw_material(const math::Vector2f &global_location, const math::Vector2f &size, const std::string &texture);

	/**
	 * @brief draw unaligned text
	 * */
	static void draw_text(const math::Vector2f &global_location, const Font *font, const std::string &text);
	
	/**
	 * @brief draw aligned text
	 * */
	static void draw_label(const math::Vector2f &global_location, const math::Vector2f &size, const Font *font, const std::string &text, const unsigned int alignment = AlignCenter);

};

} // namespace ui

#endif // __INCLUDED_UI_PAINT_H__

